"""
Functions for determining if a comment on a OSN profile appears to be abusive.
"""

import os
import enum
import enchant
import stemming.porter2
import nltk.tokenize
from doxcommentextractor.types import CommentLabel, MentionLabel

CLASSIFICATIONS = enum.Enum('not_english', 'self_comment', 'is_abuse',
                            'no_abuse_words', 'no_2nd_person')

SECOND_PERSON_WORDS = ("you", "you've", "you're", "your", "yours", "youre",
                       "youve", "you'll", "youll", "u", "ur")
SECOND_PERSON_WORDS_STEMMED = [stemming.porter2.stem(w) for w in SECOND_PERSON_WORDS]

ENG_DICT = enchant.Dict("en_US")
TWEET_TOKENIZER = nltk.tokenize.TweetTokenizer()

CURSE_WORDS_PATH = os.path.join(os.path.abspath(os.path.dirname(__file__)), '..', 'data', 'curses.txt')
with open(CURSE_WORDS_PATH, 'r') as CURSE_WORDS_HANDLE:
    CURSE_WORDS = [stemming.porter2.stem(line.strip()) for line in CURSE_WORDS_HANDLE]

def label_mention(mention):
    """Labels whether a Mention object contains abusive text.

    Args:
        mention -- A Mention object, describing a mention post in Twitter.

    Returns:
        A doxcommentextractor.MentionLabel object, describing whther the
        given mention is abusive.
    """
    is_mention_abusive = is_abusive_text(mention.text)
    return MentionLabel(mention, is_mention_abusive == CLASSIFICATIONS.is_abuse)

def label_comment(post, comment):
    """Lables whether there is abusive content in the comment left on the given
    OSN post.

    Args:
        post    -- A doxcommentextractor.types.Post object, describing a posting
                   on a OSN account.
        comment -- A doxcommentextractor.types.Comment object, describing
                   a single comment on a OSN posting.

    Returns:
        A doxcommentextractor.types.CommentLabel object, describing whether
        the given comment is abusive.
    """
    is_abuse_label = is_abuse(comment.text, comment.commenter, post.account)
    return CommentLabel(comment, is_abuse_label == CLASSIFICATIONS.is_abuse)

def is_abuse(a_text, commenter, account_holder):
    """Returns a classification of whether it looks like a comment is abusive
    content, posted by the 'commenter', to a post by the 'account_holder'.

    Args:
        a_text         -- A unicode string, depicting the text of comment
        commenter      -- A unicode string, describing the person leaving the
                          the comment.
        account_holder -- A unicode string, describing the account that
                          made the account being commented on.

    Returns:
        One of the values in the CLASSIFICATIONS enum above, describing
        whether the text looks like an abusive comment.
    """
    if commenter == account_holder:
        return CLASSIFICATIONS.self_comment

    return is_abusive_text(a_text)

def is_abusive_text(a_text):
    """Returns a classification of whether a the given text is likely to
    be an abusive OSN comment.

    Args:
        a_text -- A unicode string, depicting the text of comment

    Returns:
        One of the values in the CLASSIFICATIONS enum above, describing
        whether the text looks like an abusive comment.
    """
    # First check to see if there is a significant amount of English text
    # in the comment.  If there aren't at least two more English words
    # in the text than non-English words, assume its not English and
    # don't try to classify it further.
    num_english_words = sum([1 if ENG_DICT.check(a_word) else -1 for a_word in a_text.split()])
    if num_english_words <= 2:
        return CLASSIFICATIONS.not_english

    comment_tokens = TWEET_TOKENIZER.tokenize(a_text)
    stemmed_comment_tokens = [stemming.porter2.stem(t) for t in comment_tokens]

    # Next, check and see if there is at least one 2nd person pronoun in
    # the text.  If there isn't, we assume that its not an abusive comment
    # (or, at least not in the way we care about).
    if not any([pronoun in stemmed_comment_tokens for pronoun in SECOND_PERSON_WORDS_STEMMED]):
        return CLASSIFICATIONS.no_2nd_person

    # Next, check and see if there is at least one abusive / curse word in the
    # text.  If there are none, then its not abusive (given our measure).
    if not any([curse_word in stemmed_comment_tokens for curse_word in CURSE_WORDS]):
        return CLASSIFICATIONS.no_abuse_words

    return CLASSIFICATIONS.is_abuse
