import re

DELIMITERS = (":", ";", "|")
RE_FLAGS = re.UNICODE | re.IGNORECASE

TAGS = (
    'ONLINE:ig',
    'ONLINE:yt',
    'ONLINE:alias',
    'ONLINE:pastebin',
    'ONLINE:twitch',
    'ONLINE:skype',
    'ONLINE:fb',
    'ONLINE:twitter',
    'ONLINE:paypal',
    'ONLINE:linkedin',
    'ONLINE:snapchat',
    'ONLINE:kik',
    'ONLINE:pinterest',
    'ONLINE:aim',
    'ONLINE:gplus',
    'ONLINE:hf',
    'EMAIL',
    'ADDRESS:street',
    'ADDRESS:street2',
    'ADDRESS:city',
    'ADDRESS:state',
    'ADDRESS:zip',
    'ADDRESS:country',
    'PHONE',
    'AGE',
    'DOB',
    'NAME:first',
    'NAME:middle',
    'NAME:last',
    'DOXER:twitter',
    'DOXER:other',
    'DOXER:hf',
    'DOXER:xmpp',
    'DOXER:aim',
    'GENDER',
    'PHOTOS',
    'SSN',
)


def _split(text, splitters):
    text_bits = [text]
    for a_splitter in splitters:
        collected_parts = []
        for a_text_bit in text_bits:
            collected_parts += [t.strip() for t in a_text_bit.split(a_splitter)]
        text_bits = collected_parts
    return text_bits


def strip(text):
    tag_pattern = r'</?(' + "|".join(TAGS) + ')>'
    return re.sub(tag_pattern, '', text)


def extract(text):
    tag_pattern = r'<(?P<tag>[\w:]*)>(.*?)</(?P=tag)>'
    found_tags = {}
    matches = re.findall(tag_pattern, text, flags=RE_FLAGS)
    for tagname, body in matches:
        if tagname not in TAGS:
            continue
        try:
            if body not in found_tags[tagname]:
                found_tags[tagname].append(body)
        except KeyError:
            found_tags[tagname] = [body]
    return found_tags


def unique_tokens(tokens):
    new_tokens = []
    for a_token in tokens:
        if a_token not in new_tokens:
            new_tokens.append(a_token)
    return new_tokens


def filtered_tokens(text, indentifier, pattern, min_length=4):
    tokens = find_tokens(text, indentifier, min_length=min_length)
    return unique_tokens([t for t in tokens if re.match('^' + pattern + '$', t, flags=RE_FLAGS)])


def filter_tokens_or_urls(text, identifier, id_pattern, url_patterns):
    tokens = []
    possible_tokens = find_tokens(text, identifier)
    if isinstance(url_patterns, basestring):
        url_patterns = [url_patterns]

    for a_token in possible_tokens:
        if re.match('^' + id_pattern + '$', a_token):
            tokens.append(a_token)
            break
        for a_url_pattern in url_patterns:
            match = re.search(a_url_pattern, a_token)
            if match:
                tokens.append(match.group(1))

    return unique_tokens(tokens)


def find_tokens(text, identifier, min_length=4, splitters=None):
    if splitters is None:
        splitters = [" "]
    lower_identifier = identifier.lower()
    tokens = []

    in_text_block = False
    indentation_amount = None
    for line in text.strip().split("\n"):
        if in_text_block:
            # Sometimes tokens are listed like this:
            # Skype: a
            #        b
            #        c
            # We want to capture this kind of nesting (condition one below),
            # but w/o breaking a case like this:
            # Skype:
            #   a
            #   b
            #   c
            if indentation_amount is None or indentation_amount == 0:
                subline = line
            else:
                subline = line[indentation_amount - 1:]

            words = _split(subline.strip(), splitters)
            new_tokens = [w for w in words if len(w) >= min_length]

            if len(new_tokens) == 0:
                break

            tokens += new_tokens

            if len(tokens) == 0:
                first_possible_token = new_tokens[0]
                indentation_amount = line.find(first_possible_token)
                if indentation_amount == 0:
                    break
            continue

        # Otherwise, if we don't see the identifier (ex "Skype") on the current
        # line, then keep searching in the file...
        if not re.search(lower_identifier + ".*?[" + "".join(DELIMITERS) + "]",
                         line,
                         flags=RE_FLAGS):
            continue

        # Otherwise, the only remaining option is that the indentifier we're
        # looking for is on the current line.  We want to try and seperate
        # the identifier (ex "Skype: ") from the useful text ("example.id").
        stripping_pattern = '.*' + identifier + r"[^" + "".join(DELIMITERS) + r"]*[" + "".join(DELIMITERS) + r"]+"
        remaining_line_text = re.sub(stripping_pattern, "", line.strip(), flags=RE_FLAGS)
        words = _split(remaining_line_text.strip(), splitters)
        new_tokens = [w for w in words if len(w) >= min_length]
        if len(new_tokens) > 0:
            tokens += new_tokens
            visual_indentation_pattern = ".*" + lower_identifier + ".*?[" + "".join(DELIMITERS) + "]*\w*"
            indentation_string = re.match(visual_indentation_pattern, line, flags=RE_FLAGS).group()
            indentation_amount = len(indentation_string)
        in_text_block = True

    return tokens
